import click
import subprocess
import os
import sys
from pathlib import Path
from .config import pass_config
from .config import Config
import shellingham
from click_default_group import DefaultGroup


global_data = {
    'config': None
}

@click.group(invoke_without_command=True, cls=DefaultGroup, default='fetch', default_if_no_args=True)
@pass_config
def cli(config):
    global_data['config'] = config



from . import fetch_latest_file

@cli.command()
def install_completion():
    def setup_for_shell_generic(shell, shell_call):
        path = Path(f"/etc/{shell}_completion.d")
        NAME = shell_call.upper().replace("-", "_")
        completion = subprocess.check_output([sys.argv[0]], env={f"_{NAME}_COMPLETE": f"{shell}_source"}, shell=True)
        if path.exists():
            if os.access(path, os.W_OK):
                (path / shell_call).write_bytes(completion)
                return

        if not (path / shell_call).exists():
            rc = Path(os.path.expanduser("~")) / f'.{shell}rc'
            if not rc.exists():
                return
            complete_file = rc.parent / f'.{shell_call}-completion.sh'
            complete_file.write_bytes(completion)
            if complete_file.name not in rc.read_text():
                content = rc.read_text()
                content += '\nsource ~/' + complete_file.name
                rc.write_text(content)

    name = Path(sys.argv[0]).name
    setup_for_shell_generic(shellingham.detect_shell()[0], name)
    sys.exit(0)



import atexit
import click
import logging
from pathlib import Path
import sys
import json
import os
from contextlib import contextmanager
import configparser
from paramiko.config import SSHConfig
import subprocess

BASE_PATH = Path(os.path.expanduser("~/.fetch_latest_file.d"))
SSH_CONFIG = Path(os.path.expanduser("~/.ssh/config"))

class Config(object):
    def __init__(self):
        super().__init__()
        self.sources = {}
        self.source = None
        if BASE_PATH.exists():
            for file in BASE_PATH.glob("*"):
                if file.name.startswith('.'):
                    continue
                for section, config in self.parse_file(file):
                    self.sources[section] = config

        def cleanup():
            pass

        atexit.register(cleanup)

    def parse_file(self, file):
        config = configparser.ConfigParser()
        config.read(file)
        for section in config.sections():
            yield (section, config[section])

    @contextmanager
    def shell(self):
        config = self.get_source()

        def execute(cmd):
            output = subprocess.check_output([
                "ssh", config['host'],
            ] + cmd)
            output = output.decode('utf-8').split("\n")
            return output

        yield config, execute

    def add(self, filename, host, username, path, regex, destination):
        path = BASE_PATH / Path(filename).name
        config = configparser.ConfigParser()
        BASE_PATH.mkdir(exist_ok=True, parents=True)
        if path.exists():
            config.read(path)
        config[self.source] = {
            "host": host,
            "path": path,
            "regex": regex,
            "destination": destination,
        }
        if username:
            config[self.source]['username'] = username
        with open(path, 'w') as configfile:
            config.write(configfile)

    def get_source(self):
        if not self.source:
            raise Exception("Please define a source first!")
        return self.sources[self.source]

    def setup_logging(self):
        FORMAT = '[%(levelname)s] %(asctime)s %(message)s'
        formatter = logging.Formatter(FORMAT)
        logging.basicConfig(format=FORMAT)
        self.logger = logging.getLogger('')  # root handler
        self.logger.setLevel(self.log_level)

        stdout_handler = logging.StreamHandler(sys.stdout)
        self.logger.addHandler(stdout_handler)
        stdout_handler.setFormatter(formatter)


pass_config = click.make_pass_decorator(Config, ensure=True)



#!/usr/bin/env python3
from datetime import datetime
import re
import os
import sys
import time
import arrow
import shutil
from pathlib import Path
import json
import inquirer
import click
import subprocess
from . import cli
from .config import pass_config
from .config import Config


def get_sources(ctx, args, incomplete):
    config = Config()
    keys = config.sources.keys()
    if incomplete:
        keys = list(filter(lambda x: x.startswith(incomplete), keys))
    return keys


@cli.command()
@pass_config
def fetch(config):
    pass


def _get_files(config, shell, verbose=False):
    regex = config.get("regex", config.get("match", ".*"))
    if verbose:
        click.secho(f"Using regex '{regex}' to identify files.", fg="yellow")
    files = list(filter(bool, shell(["ls", "-tr", config["path"]])))
    for file in files:
        if re.findall(regex, file):
            yield file


def transfer(config, filename):
    destination = os.path.expanduser(config["destination"])
    source = f"{config['host']}:{config['path']}/{filename}"
    click.secho(f"Copying {source} to {destination}", fg='yellow')
    started = datetime.now()
    subprocess.run(
        ["rsync", source, destination, "-arP"]
    )
    seconds = (datetime.now() - started).total_seconds()
    click.secho(f"Success - done in {seconds}s", fg='green')


@cli.command(help="Choose specific file to download")
@click.argument("source", required=True, autocompletion=get_sources)
@pass_config
def choose(config, source):
    config.source = source
    with config.shell() as (config, shell):
        files = list(_get_files(config, shell))
        answer = inquirer.prompt(
            [inquirer.List("file", "Please choose:", choices=list(reversed(files)))]
        )
        if not answer:
            sys.exit(0)
        file = answer["file"]
        transfer(config, file)


@cli.command()
@click.option("-s", "--source", required=True)
@click.option("-f", "--filename", required=True)
@click.option("-H", "--host", required=True)
@click.option("-U", "--username", required=False)
@click.option("-r", "--regex", required=True, default=".*")
@click.option("-p", "--path", required=True)
@click.option("-d", "--destination", required=True)
@pass_config
def add(config, source, filename, host, username, regex, path, destination):
    config.source = source
    config.add(
        filename=filename,
        host=host,
        username=username,
        path=path,
        regex=regex,
        destination=destination,
    )
    click.secho("Added host.", fg="green")


@cli.command()
@pass_config
@click.argument("source", required=True, autocompletion=get_sources)
@click.option("-n", "--dryrun", is_flag=True)
@click.option("-v", "--verbose", is_flag=True)
def fetch(config, source, dryrun, verbose):
    config.source = source
    with config.shell() as (config, shell):
        files = list(_get_files(config, shell, verbose=verbose))
        if not files:
            click.secho("No files found.")
            sys.exit(1)
        file = files[-1]
        if dryrun:
            click.secho(f"File {file} would be downloaded.", fg="green")
        else:
            transfer(config, file)


@cli.command()
def sample():
    click.secho((
        "[name1]\n"
        "regex = *.dump.gz\n"
        "path = <remote path>\n"
        "host = <ssh host>\n"
        "destination = <here to put - a filename>\n"
    ))


@cli.command()
@click.option(
    "-x",
    "--execute",
    is_flag=True,
    help=("Execute the script to insert completion into users rc-file."),
)
def completion(execute):
    shell = os.environ["SHELL"].split("/")[-1]
    rc_file = Path(os.path.expanduser(f"~/.{shell}rc"))
    line = f'eval "$(_FETCH_COMPLETE={shell}_source fetch)"'
    if execute:
        content = rc_file.read_text().splitlines()
        if not list(
            filter(
                lambda x: line in x and not x.strip().startswith("#"),
                content,
            )
        ):
            content += [f"\n{line}"]
            click.secho(
                f"Inserted successfully\n{line}" "\n\nPlease restart you shell."
            )
            rc_file.write_text("\n".join(content))
        else:
            click.secho("Nothing done - already existed.")

    click.secho("\n\n" f"Insert into {rc_file}\n\n" f"echo 'line' >> {rc_file}" "\n\n")



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Note: To use the 'upload' functionality of this file, you must:
#   $ pipenv install twine --dev

import io
import os
import sys
import subprocess
from shutil import rmtree
from pathlib import Path

from setuptools.config import read_configuration
from setuptools import find_packages, setup, Command
from setuptools.command.install import install
from subprocess import check_call, check_output


setup_cfg = read_configuration("setup.cfg")
metadata = setup_cfg['metadata']
NAME = metadata['name']

# What packages are required for this module to be executed?
REQUIRED = [
    'click==8.1.3', 'inquirer', 'arrow',
    'pathlib', 'click-default-group',
    'paramiko', 'shellingham'
    ]


here = os.path.abspath(os.path.dirname(__file__))

try:
    with io.open(os.path.join(here, 'README.md'), encoding='utf-8') as f:
        long_description = '\n' + f.read()
except FileNotFoundError:
    long_description = metadata['DESCRIPTION']

# Load the package's __version__.py module as a dictionary.
about = {}
if not metadata['version']:
    project_slug = metadata['name'].lower().replace("-", "_").replace(" ", "_")
    with open(os.path.join(here, project_slug, '__version__.py')) as f:
        exec(f.read(), about)
else:
    about['__version__'] = metadata['version']


setup(
    version=about['__version__'],
    long_description=long_description,
    long_description_content_type='text/markdown',
    packages=find_packages(exclude=["tests", "*.tests", "*.tests.*", "tests.*"]),
    install_requires=REQUIRED,
    include_package_data=True,
)
